use super::super::super::repositories;
use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use diesel::{
  r2d2::{ConnectionManager, Pool},
  PgConnection,
};
use prost::Message;
use std::str::FromStr;

pub struct EventHandler {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
  pool: Pool<ConnectionManager<PgConnection>>,
  member_balance_repository: repositories::member_balance::repository::Repository,
}

impl std::fmt::Debug for EventHandler {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("EventHandler of members").finish()
  }
}

impl EventHandler {
  ///
  pub fn new(
    connection_broker: nats::asynk::Connection,
    queue_broker: String,
    pool: Pool<ConnectionManager<PgConnection>>,
  ) -> EventHandler {
    EventHandler {
      connection_broker,
      queue_broker,
      pool,
      member_balance_repository: repositories::member_balance::repository::Repository::new(),
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(self.event_after_update_member_balance()).map(|_| ())
  }

  async fn event_after_update_member_balance(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member_balance::EVENT_SUBJECT_AFTER_UPDATE_MEMBER_BALANCE,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let eve =
          bpr::ss::member_balance::AfterUpdateMemberBalanceEvent::decode(message.data.as_slice())
            .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match eve.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let event = match eve.event {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid event information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "event".to_string(),
                  param: "event".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let member_id = uuid::Uuid::from_str(event.member_id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid member_id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "member_id".to_string(),
              value: event.member_id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let id = match self
          .member_balance_repository
          .select_by_member_id(&conn, member_id)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })? {
          Some(_mb) => _mb.id,
          None => {
            let _mb = self
              .member_balance_repository
              .insert(
                &conn,
                &repositories::member_balance::models::NewMemberBalance { member_id },
              )
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })?;

            _mb.id
          }
        };

        self
          .member_balance_repository
          .update(
            &conn,
            id,
            &repositories::member_balance::models::ModifyMemberBalance {
              balance: event.balance,
              balance_bota: event.balance_bota,
              balance_sum: event.balance_sum,
            },
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        println!("error: {}", e);
      }
    }

    Ok(())
  }
}
