//!
//!
use super::{models, schema::member_sessions};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of member_sessions").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_session: &models::NewMemberSession,
  ) -> Result<models::MemberSession, Error> {
    let captcha = diesel::insert_into(member_sessions::table)
      .values(new_session)
      .get_result::<models::MemberSession>(conn)?;

    Ok(captcha)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::MemberSession>, Error> {
    match member_sessions::table
      .find(id as uuid::Uuid)
      .first::<models::MemberSession>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::MemberSession>, Error> {
    use member_sessions::dsl;

    match member_sessions::table
      .filter(dsl::member_id.eq(member_id))
      .first::<models::MemberSession>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = member_sessions::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_sessions::dsl::member_id.eq(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::MemberSession>, Error> {
    let mut q = member_sessions::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_sessions::dsl::member_id.eq(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::pagination::Sort::ASC(property) => match property.as_str() {
            "member_id" => {
              q = q.order_by(member_sessions::member_id.asc());
            }
            "last_accessed_at" => {
              q = q.order_by(member_sessions::last_accessed_at.asc());
            }
            "expires_at" => {
              q = q.order_by(member_sessions::expires_at.asc());
            }
            "created_at" => {
              q = q.order_by(member_sessions::created_at.asc());
            }
            _ => {}
          },
          bcr::pagination::Sort::DESC(property) => match property.as_str() {
            "member_id" => {
              q = q.order_by(member_sessions::member_id.desc());
            }
            "last_accessed_at" => {
              q = q.order_by(member_sessions::last_accessed_at.desc());
            }
            "expires_at" => {
              q = q.order_by(member_sessions::expires_at.desc());
            }
            "created_at" => {
              q = q.order_by(member_sessions::created_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::MemberSession>(conn)
  }

  ///
  pub fn update_last_access(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifyMemberSessionForLastAccess,
  ) -> Result<u64, Error> {
    use member_sessions::dsl;

    diesel::update(dsl::member_sessions.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: uuid::Uuid) -> Result<u64, Error> {
    use member_sessions::dsl;

    diesel::delete(member_sessions::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete_expired(&self, conn: &diesel::PgConnection) -> Result<u64, Error> {
    use member_sessions::dsl;

    let now = chrono::Utc::now().timestamp();

    diesel::delete(dsl::member_sessions.filter(dsl::expires_at.le(now)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
